# coding: utf-8

import textwrap
from colorama import init
from termcolor import colored
import sys
import coloredlogs, logging

class Messages():
    init()

    def msg(self,str_msg,context_str=None):
        msg = colored(str_msg,'white')
        print('\n'+msg)

    def dbg(self,str_msg):
        final_msg = colored('[DEBUG]: ','magenta',attrs=['bold'])
        final_msg += colored(str_msg,'white')
        print('\n'+final_msg)

    def warn(self,str_msg):
        final_msg = colored('WARNING: ','yellow',attrs=['bold'])
        final_msg += colored(str_msg,'white')
        print('\n'+final_msg)


    def error(self,str_msg):
        final_msg = colored('ERROR: ','red',attrs=['bold'])
        final_msg += colored(str_msg,'red')
        print('\n'+final_msg)

    def critical(self,str_msg):
        final_msg = colored('ERROR: ','red',attrs=['bold'])
        final_msg += colored(str_msg+'\nExiting program...','red')
        print('\n'+final_msg)
        sys.exit()

    def respond(self,str_msg):
        final_msg = colored(str_msg,'green')
        print('\t'+final_msg)

    def headline(self,str_msg):
        str_msg = str_msg.upper()
        final_msg = colored(str_msg,'blue',attrs=['bold','underline'])
        print('\t'+final_msg)


def parse_func_list(func_lst, identify_lst, ignore_keywords = []):
    """
    Reads through a list of available methods and retrieves the methods that have
    matching words in the identify list, then it removes those key words and Returns
    a dictionary of simplified method names with values as their corresponding
    method

    Args:
        func_list: A list of function names and functions
        identify_lst: A list of keywords to match with
        ignore_keywords: List contain keywords that will ignore functions with that keyword
    Returns:
        options: A dictionary that has keys that are simplified names of the methods, and values are the methods
    """
    options = {}
    ignore_keywords = [w.lower() for w in ignore_keywords]
    for f_name,fn in func_lst:
        #Return a list of true when a word is found
        words_found = [True for w in identify_lst if w in f_name]
        #If the number of the matches matches the number of keywords were looking for
        if len(words_found) == len(identify_lst) and 'RAD' not in f_name:
            #Remove the keywords to form the simplified method name
            name = f_name
            for word in identify_lst:
                #print(f_name,word)
                name = name.replace(word,'')
            ignores = [True for ign in ignore_keywords if ign in name.lower()]
            if len(ignores) == 0:
                options[name.lower()]=fn

    return options

def parse_help(help_str):
    """
    Auto parses the help dialogs provided by developers in the doc strings.
    This is denoted by helpme - text and then a blank line.

    e.g.

    Function doc string
    helpme - more end user friendly message about what this will do

    my arguments...

    """
    result = None

    if help_str != None:
        if 'helpme' in help_str:
            z = help_str.split('helpme')
            result = z[-1].split('-')[-1].strip()

    return result

def columnize_str(entrie,width):
    """
    Takes a long string that cannot be formatted correctly into a column
    """
    pass

def print_helpme(help_str,help_dict):
    """
    Trys to help the user on request for help.
    """
    out_str = ""
    show_all = False
    no_help = 0
    no_doc = False
    # User provides specific help request
    if help_dict != None:
        if "-" in help_str:
            h = help_str.split("-")
            k = ("".join(h[1:]).strip()).lower()
            if k in help_dict:
                formatted = textwrap.fill(long_str,width = width)

                for i,line in enumerate(formatted.split('\n')):
                    if i != 0:
                        k = ""
                    result += "\n{0:<20} {1:<20}\n".format(k,v)
            else:
                show_all = True
        else:
            show_all= True

        if show_all:
            for k,v in help_dict.items():
                if v == None:
                    no_help +=1
                    v = ('No help documentation.')

                out_str +='\n{0:<25} {1:<25}\n'.format(k,v)
        if no_help == len(help_dict.keys()):
            no_doc = True
    else:
        no_doc = True

    if no_doc:
        out_str = ('\nNo help documentation.\nPlease email micah@adventuredata.com\n')
        print(help_dict)
    t = '\n{0:<20} {1:<20}\n'.format('OPTIONS','DESCRIPTION')
    print_able='\n{0}'.format(colored(t,'magenta',attrs=['bold']))
    print_able+=out_str
    print(print_able)

def get_logger(name, level='DEBUG', ext_logger=None,):
    """

    """
    fmt = fmt='%(name)s %(levelname)s %(message)s'
    if ext_logger == None:
        log = logging.getLogger(name)
    else:
        log = ext_logger

    coloredlogs.install(fmt=fmt,level=level, logger=log)
    return log



# coding: utf-8

import sys
import binascii
import serial
import time
import inspect
from radicl.api import RAD_API
from radicl import serial as rs
from radicl.ui_tools import get_logger
from radicl import __version__
from radicl.ui_tools import Messages, parse_func_list, print_helpme, parse_help

import struct
import datetime


class RAD_Probe():
    """
    Class for directly interacting with the probe.
    """
    def __init__(self, ext_api=None):
        """
        Args:
            ext_api: rad_api.RAD_API object preinstantiated
        """

        self.log = get_logger(__name__, level='DEBUG')

        # Check if an external API object was passed in.
        if (ext_api != None):
            # An external API object was provided. Use it. Note: We assume here
            # that the appropriate initialization, identification, and API port
            # enable procedure has already taken place
            self.api = ext_api
        else:
            # No external API oject was provided. Create new serial and API
            # objects for internal use
            port = rs.RAD_Serial()
            port.openPort()

            if not port:
        	    self.log.info("No device present")
            else:
                port.flushPort()
                # Create the API and FMTR instances The API class is linked to
                # the port object
                api = RAD_API(port)

                #Switch the device over to API mode
                api.sendApiPortEnable()
                self.api = api

                #Delay a bit and then identify the attached device
                time.sleep(0.5)
                ret = api.Identify()
                time.sleep(0.5)

                # Manages the settings
                settings_funcs = inspect.getmembers(self.api,
                                                    predicate=inspect.ismethod)
                self.settings = parse_func_list(settings_funcs,['Meas','Set'])
                self.getters = parse_func_list(settings_funcs,['Meas','Get'])

    #Generic read function
    def __readData(self, buffer_ID):
        """
        Prive function to retrieve data from the probe.
         Args:
            buffer_ID:
        """
        num_segments = 0
        data = []
        ret1 = self.api.MeasGetNumSegments(buffer_ID)
        # Returns a positive status
        if (ret1['status'] == 1):
            num_segments = int.from_bytes(ret1['data'],byteorder='little')

            if (num_segments != 0 and num_segments != None):
                self.log.debug("Reading %d segments" % num_segments)
                byte_counter = 0

                for ii in range(1, (num_segments+1)):
                    ret2 = self.api.MeasReadDataSegment(buffer_ID, (ii-1))

                    if (ret2['status'] == 1):
                        # Data segment read successfull

                        if (ret2['data'] != None):
                            byte_counter = byte_counter + len(ret2['data'])
                            data_chunk = ret2['data']
                            data.extend(data_chunk)

                    # Data segment read failed. Retry
                    else:
                        wait_time = 0.005

                        for jj in range(0, 10):
                            time.sleep(wait_time)
                            wait_time = wait_time * 2
                            ret3 = self.api.MeasReadDataSegment(buffer_ID, ii)

                            if (ret3['status'] == 1):
                                byte_counter = byte_counter + len(ret3['data'])

                                data_chunk = ret3['data']
                                data.extend(data_chunk)
                                break

                            elif (ret3['errorCode'] != None):
                                self.log.error("readSegmentData error: %d (Retry %d, "
                                      "Segment=%d, buffer_ID=%d)" % \
                                      (ret3['errorCode'], jj, ii, buffer_ID))

                            else:
                                self.log.error("readSegmentData error: COM "
                                                "(Retry %d, Segment=%d, "
                                                "buffer_ID=%d)" % \
                                                (jj, ii, buffer_ID))

                        # Retry failed! Return here
                        if ((ret3['status'] != 1) or (ret3['data'] == None) ):
                            return {'status': 0,
                                    'SegmentsAvailable': num_segments,
                                    'SegmentsRead': ii,
                                    'BytesRead': byte_counter,
                                    'Data': None}
                # -- END OF FOR LOOP --
                # Read process has successfully completed
                return {'status': 1, 'SegmentsAvailable': num_segments,
                                     'SegmentsRead': ii,
                                     'BytesRead': byte_counter,
                                     'Data': data}
            else:
                self.log.error("Data read error: No data segments available")

        # No error code provided
        elif (ret1['errorCode'] != None):
            self.log.error("getNumSegments error: %d (buffer_ID = %d)" % \
            (ret1['errorCode'], buffer_ID))
        else:
            self.log.error("getNumSegments error: COM")
        return {'status': 0, 'SegmentsAvailable': 0, 'SegmentsRead': 0,
                                                     'BytesRead': 0,
                                                     'Data': None}

    # ********************
    # * PUBLIC FUNCTIONS *
    # ********************
    def getProbeSerial(self):
        """
        Returns the probe's serial number. The return value is a string. If the
        request fails it will return None
        """

        ret = self.api.getSerialNumber()

        if ( (ret['status'] == 1) and (ret['data'] != None) ):
            return ret['data'].decode("utf-8")
        else:

            if (ret['errorCode'] != None):
                self.log.error("getProbeSerial error: %d" % ret['errorCode'])

            else:
                self.log.error("getProbeSerial error: COM")

            return None

    def getProbeSystemStatus(self):
        """
        Returns the probe's system status. The return value is an integer.
        If the request fails it will return None
        """

        ret = self.api.getSystemStatus()

        if ( (ret['status'] == 1) and (ret['data'] != None) ):
            return int.from_bytes(ret['data'], byteorder='little')

        else:

            if (ret['errorCode'] != None):
                self.log.error("getProbeSystemStatus error: %d" % \
                                                              ret['errorCode'])

            else:
                self.log.error("getProbeSystemStatus error: COM")

            return None

    def getProbeRunState(self):
        """
        Returns the probe's run state. The return value is an integer.
        If the request fails it will return None
        """

        ret = self.api.getRunState()

        if ((ret['status'] == 1) and (ret['data'] != None)):
            return int.from_bytes(ret['data'], byteorder='little')

        else:

            if (ret['errorCode'] != None):
                self.log.error("getProbeRunState error: %d" % ret['errorCode'])

            else:
                self.log.error("getProbeRunState error: COM")

            return None

    def getProbeMeasState(self):
        """
        Retrieves the probe measurement state and converts it to an integer

        Returns:
            integer-measurement state of the probe, or none if error arises.
        """
        ret = self.api.getMeasState()
        if ( (ret['status'] == 1) and (ret['data'] != None) ):
            return int.from_bytes(ret['data'],byteorder ='little')
        else:
            if (ret['errorCode'] != None):
                self.log.error("getState error: %d" % ret['errorCode'])
            else:
                self.log.error("getState error: COM")
            return None

    def startMeasurement(self):
        """
        Starts a new measurement. Returns 1 if successfull, 0 otherwsie
        """

        ret = self.api.MeasStart()
        self.log.debug("Start Measurement Reqested.")

        if (ret['status'] == 1):
            self.wait_for_state(1)
            self.log.info("Measurement Started...")

            return 1

        else:
            if (ret['errorCode'] != None):
                self.log.error("measStart error: %d" % ret['errorCode'])

            else:
                self.log.error("measStart error: COM")

            return 0

    def stopMeasurement(self):
        """
        Stops an ongoing measurement. Returns 1 if successfull, 0 otherwise
        """

        ret = self.api.MeasStop()
        self.log.debug("Stop Measurement Reqested.")

        if (ret['status'] == 1):
            self.log.info("Measurement stopped...")

            return 1
        else:
            if (ret['errorCode'] != None):
                self.log.error("measStop error: %d" % ret['errorCode'])
            else:
                self.log.error("measStop error: COM")
            return 0

    def resetMeasurement(self):
        """
        Resets the measurement FSM to prepare for a new measurement.
        Returns 1 if successfull, 0 otherwise
        """

        ret = self.api.MeasReset()

        if (ret['status'] == 1):
            self.wait_for_state(5)

            return 1
        else:
            if (ret['errorCode'] != None):
                self.log.error("measStop error: %d" % ret['errorCode'])

            else:
                self.log.error("measStop error: COM")

            return 0

    def wait_for_state(self, state, retry = 10):
        """
        Waits for the specifed state to occur. This is particularly useful when
        a command is requested.
        """

        attempts = 0
        pstate = None
        result = False
        self.log.debug("Waiting for state {0}".format(state))

        while not result:
            pstate = self.getProbeMeasState()
            result = pstate==state

            if attempts > retry:
                self.log.error("Retry Exceeded waiting for state {0}".format(state))
                result = False
                break
            else:
                attempts +=1

            time.sleep(0.1)
        return result

    def readRawSensorData(self):
        """
        Reads the RAW sensor data.
        helpme - Raw NIR data for 4 sensors

        Returns:
            dict - containing data or None if read failed
        """
        ret = self.__readData(0)

        # Sucessfully read data
        if (ret['status'] == 1):
            #***** DATA INTEGRITY CHECK *****
            if (ret['SegmentsAvailable'] != ret['SegmentsRead']):
                # Data integrity error (not all segments read)
                self.log.error("readRawSensorData error: Data integrity error (not all "
                      "segments read)")
                return None
            expected_bytes = ret['SegmentsRead'] * 256
            if (expected_bytes != ret['BytesRead']):
                # Data integrity error (not all bytes read - incomplete segment)
                # For the raw sensor data, this is also the check to ensure we have an even amount of bytes to break into sensor pairs
                self.log.error("readRawSensorData error: Data integrity error (not all "
                      "bytes read - incomplete segment)")
                self.log.error("Expected=%d, Read=%d" % (expected_bytes, ret['BytesRead']))
                return None

            #***** DATA PARSING *****
            data = ret['Data']
            sensor1 = []
            sensor2 = []
            sensor3 = []
            sensor4 = []
            total_runs = expected_bytes // 8
            offset = 0

            for ii in range(0, total_runs):
                sensor1.append(data[offset] + data[(offset + 1)] * 256)
                sensor2.append(data[(offset + 2)] + data[(offset + 3)] * 256)
                sensor3.append(data[(offset + 4)] + data[(offset + 5)] * 256)
                sensor4.append(data[(offset + 6)] + data[(offset + 7)] * 256)
                offset = offset + 8
            return {'Sensor1': sensor1, 'Sensor2': sensor2, 'Sensor3': sensor3,
                    'Sensor4': sensor4}

        # Read failed!
        else:
            return None

    def readRawAccelerationData(self):
        """
        Reads the RAW acceleration data
        helpme - Accelerometer data from the probe

        Returns:
            dict: containing accel data (x,y,z) or None if read failed


        """
        ret = self.__readData(1)

        # Sucessfully read data
        if (ret['status'] == 1):
            #***** DATA INTEGRITY CHECK *****
            if (ret['SegmentsAvailable'] != ret['SegmentsRead']):
                # Data integrity error (not all segments read)
                self.log.error("readRawAccelerationData error: Data integrity error (not"
                      " all segments read)")
                return None

            total_bytes = ret['BytesRead']

            # Data integrity error (not all bytes read - incomplete data segment)
            if ( (total_bytes % 6) != 0 ):
                # The data set is not an integer multiple of 6 (2 bytes per
                # axis, 3 axes total)
                self.log.error("readRawAccelerationData error: Data integrity error "
                      "(incomplete data set)")
                return None

            #***** DATA PARSING *****
            data = ret['Data']
            x_axis = []
            y_axis = []
            z_axis = []
            total_runs = total_bytes // 6
            offset = 0

            for ii in range(0, total_runs):
                x_axis.append((data[(offset + 0)] + (data[(offset + 1)] * 256) ) / 1000)
                y_axis.append((data[(offset + 2)] + (data[(offset + 3)] * 256) ) / 1000)
                z_axis.append((data[(offset + 4)] + (data[(offset + 5)] * 256) ) / 1000)
                offset = offset + 6
            return {'X-Axis': x_axis, 'Y-Axis': y_axis, 'Z-Axis': z_axis}
        # Read failed!
        else:
            return None

    def readAccelerationCorrelationData(self):
        """
        Reads the acceleration correllation data
        """

        ret = self.__readData(5)
        if (ret['status'] == 1):
            #Sucessfully read data
            #***** DATA INTEGRITY CHECK *****
            # Data integrity error (not all segments read)
            if (ret['SegmentsAvailable'] != ret['SegmentsRead']):
                self.log.error("readAccelerationCorrelationData error: Data integrity "
                      "error (not all segments read)")
                return None
            total_bytes = ret['BytesRead']
            if ( (total_bytes % 4) != 0 ):
                # Data integrity error (not all bytes read - incomplete data segment)
                # The data set is not an integer multiple of 4 (Correlation number is a 32-bit int => 4 bytes)
                self.log.error("readAccelerationCorrelationData error: Data integrity "
                      "error (incomplete data set)")
                return None

            #***** DATA PARSING *****
            data = ret['Data']
            correll_data = []
            total_runs = total_bytes // 4
            offset = 0

            for ii in range(0, total_runs):
                correll_data.append(data[(offset + 0)] + (data[(offset + 1)] * 256) + (data[(offset + 2)] * 65536) + (data[(offset + 3)] * 16777216))
                offset = offset + 4
            return correll_data
        else:
            #Read failed!
            return None

    def readRawPressureData(self):
        """
        Reads the RAW pressure data, including the correlation index
        """

        ret = self.__readData(2)

        # Sucessfully read data
        if (ret['status'] == 1):
            # ***** DATA INTEGRITY CHECK *****
            if (ret['SegmentsAvailable'] != ret['SegmentsRead']):
                # Data integrity error (not all segments read)
                self.log.error("readRawPressureData error: Data integrity error (not "
                      "all segments read)")
                return None

            # Data integrity error (not all bytes read - incomplete data segment)
            total_bytes = ret['BytesRead']
            if ((total_bytes % 3) != 0):
                # The data set is not an integer multiple of 3 (Raw pressure is in 24-bit format => 3 bytes)
                self.log.error("readRawPressureData error: Data integrity error (incomplete data set)")
                return None

            # ***** DATA PARSING *****
            data = ret['Data']
            pressure_data = []
            total_runs = total_bytes // 3
            offset = 0
            for ii in range(0, total_runs):
                this_value = data[(offset + 0)] + (data[(offset + 1)] * 256) + \
                            (data[(offset + 2)] * 65536)

                pressure_data.append(this_value/4096)
                offset = offset + 3
            return pressure_data

        # Read failed!
        else:
            return None

    def readDepthData(self):
        """
        Reads the converted depth data, including the correlation index
        Return type is a dict containing data or None if read failed
        """
        ret = self.__readData(3)
        if (ret['status'] == 1):
            # Sucessfully read data
            # ***** DATA INTEGRITY CHECK *****
            if (ret['SegmentsAvailable'] != ret['SegmentsRead']):
                # Data integrity error (not all segments read)
                self.log.error("readDepthData error: Data integrity error (not all "
                      "segments read)")
                return None

            total_bytes = ret['BytesRead']
            if ((total_bytes % 4) != 0):
                # Data integrity error (not all bytes read - incomplete data segment)
                # The data set is not an integer multiple of 4 (Floatingpoint
                #values are 32-bit long => 4 bytes)
                self.log.error("readDepthData error: Data integrity error "
                      "(incomplete data set)")
                return None

            # ***** DATA PARSING *****
            data = ret['Data']
            depth_data = []
            total_runs = total_bytes // 4
            offset = 0
            for ii in range(0, total_runs):
                this_byte_list = data[(offset + 0):(offset + 4)]
                this_byte_object = bytes(this_byte_list)
                this_value = struct.unpack('f', this_byte_object)
                depth_data.append(this_value)
                offset = offset + 4
            return depth_data
        else:
            # Read failed!
            return None

    def readFilteredDepthData(self):
        """
        Retrieves the filtered depth data according to the zero-phase low-pass
        filter (equivalent to Matlab's 'filtfilt'). The amount of filtering
        applied is set by :func:`~radicl.RAD_API.MeasSetZPFO`

        helpme - The probe's filtered depth data is the filtered depth
        calculated from barometer data where the amount of filtering is set by
        the ZPFO option under settings.

        """

        ret = self.__readData(4)
        if (ret['status'] == 1):
            # Sucessfully read data
            # ***** DATA INTEGRITY CHECK *****
            if (ret['SegmentsAvailable'] != ret['SegmentsRead']):
                # Data integrity error (not all segments read)
                self.log.error("readFilteredDepthData error: Data integrity error "
                      "(not all segments read)")
                return None

            total_bytes = ret['BytesRead']
            if ((total_bytes % 4) != 0):
                # Data integrity error (not all bytes read - incomplete data segment)
                # The data set is not an integer multiple of 4 (Floatingpoint values are 32-bit long => 4 bytes)
                self.log.error("readFilteredDepthData error: Data integrity error "
                      "(incomplete data set)")
                return None

            # ***** DATA PARSING *****
            data = ret['Data']
            depth_data = []
            total_runs = total_bytes // 4
            offset = 0

            for ii in range(0, total_runs):
                this_byte_list = data[(offset + 0):(offset + 4)]
                this_byte_object = bytes(this_byte_list)
                this_value = struct.unpack('f', this_byte_object)
                depth_data.append(this_value)
                offset = offset + 4
            return depth_data

        else:
            # Read failed!
            return None

    def readPressureDepthCorrelationData(self):
        """
        Reads the pressure/depth correllation data
        """

        ret = self.__readData(6)
        if (ret['status'] == 1):
            #Sucessfully read data
            #***** DATA INTEGRITY CHECK *****
            if (ret['SegmentsAvailable'] != ret['SegmentsRead']):
                #Data integrity error (not all segments read)
                self.log.error("readPressureDepthCorrelationData error: Data integrity "
                      "error (not all segments read)")
                return None

            total_bytes = ret['BytesRead']
            if ( (total_bytes % 4) != 0 ):
                # Data integrity error (not all bytes read - incomplete data segment)
                # The data set is not an integer multiple of 4 (Correlation number is a 32-bit int => 4 bytes)
                self.log.error("readPressureDepthCorrelationData error: Data integrity "
                      "error (incomplete data set)")
                return None

            #***** DATA PARSING *****
            data = ret['Data']
            correll_data = []
            total_runs = total_bytes // 4
            offset = 0

            for ii in range(0, total_runs):
                correll_data.append(data[(offset + 0)] + (data[(offset + 1)] * \
                                    256) + (data[(offset + 2)] * 65536) + \
                                    (data[(offset + 3)] * 16777216))
                offset = offset + 4
            return correll_data

        else:
            #Read failed!
            return None

    def readSensorDepthCombo(self):
        """
        Reads the processed data as a sensor-depth combo
        """

        ret = self.__readData(7)
        if (ret['status'] == 1):
            #Sucessfully read data
            #***** DATA INTEGRITY CHECK *****
            if (ret['SegmentsAvailable'] != ret['SegmentsRead']):
                #Data integrity error (not all segments read)
                self.log.error("readSensorDepthCombo error: Data integrity error (not "
                      "all segments read)")
                return None

            total_bytes = ret['BytesRead']
            if ( (total_bytes % 16) != 0 ):
                # Data integrity error (not all bytes read - incomplete data segment)
                # The data set is not an integer multiple of 4 (Correlation number is a 32-bit int => 4 bytes)
                self.log.error("readPressureDepthCorrelationData error: Data integrity"
                      " error (incomplete data set)")
                return None

            self.log.info("Total Datapoints = %d" % ret['SegmentsRead'])
            #***** DATA PARSING *****
            data = ret['Data']
            total_runs = total_bytes // 16
            offset = 0
            transaction_id = []
            sensor1 = []
            sensor2 = []
            sensor3 = []
            sensor4 = []
            depth = []

            for ii in range(0, total_runs):
                this_data_set = data[offset: (offset + 16)]
                transaction_id.append(this_data_set[0] + (this_data_set[1] * \
                                      256) + (this_data_set[2] * 65536) + \
                                      (this_data_set[3] * 16777216))
                depth.append(this_data_set[4] + (this_data_set[5] * 256) + \
                             (this_data_set[6] * 65536) + (this_data_set[7] * \
                             16777216))
                sensor1.append(this_data_set[8] + (this_data_set[9] * 256))
                sensor2.append(this_data_set[10] + (this_data_set[11] * 256))
                sensor3.append(this_data_set[12] + (this_data_set[13] * 256))
                sensor4.append(this_data_set[14] + (this_data_set[15] * 256))
                offset = offset + 16
            return {'ID': transaction_id, 'Sensor1': sensor1,
                                          'Sensor2': sensor2,
                                          'Sensor3': sensor3,
                                          'Sensor4': sensor4,
                                          'Depth': depth}
        else:
            #Read failed!
            return None

    def readMeasurementTemperature(self):
        """
        Reads the temperature from the last measurement
        """

        ret = self.api.MeasGetMeasTemp()
        if (ret['status'] == 1):
            data = ret['data']
            this_byte_object = bytes(data)
            this_value = struct.unpack('i', this_byte_object)
            return this_value

        elif (ret['errorCode'] != None):
            self.log.error("getMeasTemp error: %d" % ret['errorCode'])
        else:
            self.log.error("getMeasTemp error: COM")
        return None

    def getZPFO(self):
        """
        Reads the probes zpfo
        """

        ret = self.api.MeasGetZPFO()
        if (ret['status'] == 1):
            data = ret['data']
            value = int.from_bytes(data,byteorder = 'little')
            return value

        elif (ret['errorCode'] != None):
            self.log.error("getZPFO error: %d" % ret['errorCode'])
        else:
            self.log.error("getZPFO error: COM")
        return None

    def getProbeHeader(self):
        """
        Returns the lines that should be in every data file produced.
        """
        t = datetime.datetime.now()
        fstr = "{0}-{1:02d}-{2:02d}--{3:02d}:{4:02d}:{5:02d}"
        time_stamp = fstr.format(t.year,t.month,t.day,
                                 t.hour,t.minute,t.second)

        header = "RECORDED={0}\n"
        header += "radicl VERSION={1}\n"
        header += "FIRMWARE REVISION={2}\n"
        header += "HARDWARE REVISION={3}\n"
        header += "MODEL NUMBER={4}\n"
        #header += "SERIAL NUMBER={5}\n"

        final = header.format(time_stamp,
                              __version__,
                              self.api.fw_rev,
                              self.api.hw_rev,
                              self.api.hw_id,
                              )
        return final

    def getSetting(self,**kwargs):
        """
        Reads the probes setting
        helpme - Zero phase filter order for smoothing the depth data

        """
        setting_name = kwargs['setting_name']
        if setting_name == 'calibdata':
            ret = self.getters[setting_name](kwargs['sensor'])

        else:
            ret = self.getters[setting_name]()

        if (ret['status'] == 1):
            data = ret['data']
            value = int.from_bytes(data,byteorder = 'little')
            return value
        elif (ret['errorCode'] != None):
            out.error("get{0} error: {1}".format(setting_name, ret['errorCode']))
            return None

        else:
            out.error("get{0} error: COM".format(setting_name))
            return None

    def setSetting(self,**kwargs):
        """
        sets the probe's setting
        """
        setting_name = kwargs['setting_name']

        if setting_name == 'calibdata':
            ret = self.settings[setting_name](kwargs['sensor'],
                                              kwargs['hi_value'])

        else:
            ret = self.settings[setting_name](kwargs['value'])

        if (ret['status'] == 1):
            return True
        elif (ret['errorCode'] != None):
            out.error("get{0} error: {1}".format(setting_name, ret1['errorCode']))
        else:
            out.error("get{0} error: COM".format(setting_name))
        return None



# coding: utf-8

import sys
import binascii
import serial
import time
from radicl import serial as rs
from radicl.ui_tools import get_logger

pca_id_list = ["UNKNOWN", "PB1", "PB2", "PB3"]

class RAD_API():
	"""
	Class for directly interacting with the probe in a non-human friendly way
	"""

	def __init__(self, port):
		self.port = port
		self.log = get_logger(__name__, level='DEBUG')

	def __sendCommand(self, data):
		"""
		Generic send function
		Returns 1 of successfull, 0 otherwise
		"""

		success = 0
		try:
			#Send the data
			self.port.writePort(data)

		except Exception as e:
			self.log.error(e)

		else:
			success = 1

		finally:
			return success

	def __getResponse(self):
		"""
		Generic read function
		Returns 1 if successfull, 0 otherwise
		"""

		success = 0
		response = ""

		try:
			num_bytes_in_buffer = self.port.numBytesInBuffer()
			if (num_bytes_in_buffer > 0):
				response = self.port.readPort(num_bytes_in_buffer)

		except Exception as e:
			self.log.error(e)

		else:
			success = 1

		finally:
			if (success):
				return response
			else:
				return None

	def __send_receive(self, data, read_delay=0.05):
		"""
		Generic send/receive function
		Returns the response if successfull, empty result otherwise
		"""

		delay_counter = 0
		#Make sure that the read delay is at least 1ms (i.e. it cannot be 0)
		if (read_delay < 0.001):
			read_delay = 0.001
		ret = self.__sendCommand(data)

		if (ret):
			#The command was successfully sent. Now read the response
			time.sleep(0.001)
			ret = self.__getResponse()
			# Read the response as long as we don't get anything back AND
			# we haven't exceeded the max. read delay
			while (((ret == None) or (ret == "")) and (delay_counter < read_delay)):
				time.sleep(0.001)
				delay_counter = delay_counter + 0.001
				ret = self.__getResponse()
			# If we get here either we have exceeded the max. read delay, or
			#  we have received a response.
			return ret

		else:
			#There was an issue with sending the command
			return None

	def __isACK(self, message, cmd=None):
		"""
		Returns 1 if the message contains an ACK
		"""
		if (len(message) >= 5):
			#If a command was specified, check that it matches the message
			if (cmd != None):
				if (message[1] != cmd):
					return 0
			#If we get here then we have passed the additional specified checks.
			if (message[2] == 0x04):
				#ACK detectec
				return 1
			else:
				#No ACK found
				return 0
		else:
			#Message is not long enough. Indicate an error
			return 0

	def __isNACK(self, message, cmd=None):
		"""
		Returns 1 if the message contains a NACK
		"""

		if (len(message) >= 5):

			# If a command was specified, check that it matches the message
			if (cmd != None):
				if (message[1] != cmd):
					return 0

			# If we get here then we have passed the additional user specified checks.
			if (message[2] == 0x05):
				# NACK detectec
				return 1
			else:
				# No NACK found
				return 0
		else:
			# Message is not long enough. Indicate an error
			return 0

	def __getNACKValue(self, message):
		"""
		Returns the NACK message value
		If an error is detected, this will simply return 0
		"""

		if (len(message) >= 7):
			nack_val = message[5:7]
			return int.from_bytes(nack_val, byteorder='little')

		else:
			return 0;

	def __isResponse(self, message, cmd=None, data_len=None):
		"""
		Returns 1 if the message is a valid response
		0 for data_len means that the payload length could be variable
		"""

		length = len(message)
		if (length >= 5):
			if (message[2] == 0x02):
				calc_len = message[4] + 5
				# If a command was specified we need to check if it matches.
				# If it is incorrect we will return immediately
				if (cmd != None):
					if (message[1] != cmd):
						return 0
				# If a data length was specified we need to check if it
				# matches the message's data payload length. If it is incorrect we will return immediately
				if ((data_len != None) and (data_len != 0)):
					if (message[4] != data_len):
						return 0
				# If we get here then we have passed the additional
				# specified checks. Finally, check if the overall length
				# matches (integrity check)
				if (length == calc_len):
					#The length is correct
					return 1
				else:
					#Not a response or incorrect length
					return 0
			elif ( (message[2] == 0x06) and (length == 261) ):
				return 1
			else:
				return 0
		else:
			return 0

	def __getNumPayloadBytes(self, message):
		"""
		Returns the number of bytes present in the payload. This takes long #
		responses into account
		"""

		if (message[2] == 0x02):
			return message[4]
		elif (message[2] == 0x06):
			return 256
		else:
			return 0

	def __isPushMessage(self, message, cmd=None, data_len=None):
		"""
		Returns 1 if the message is a valid push message/response
		"""
		length = len(message)
		if (length >= 5):
			calc_len = message[4] + 5
			# If a command was specified we need to check if it matches.
			# If it is incorrect we will return immediately
			if (cmd != None):
				if (message[1] != cmd):
					return 0
			#If a data length was specified we need to check if it matches the
			#  message's data payload length. If it is incorrect we will
			# return immediately
			if (data_len != None):
				if (message[4] != data_len):
					return 0
			#If we get here then we have passed the additional user specified
			# checks. Finally, check if the message is a response and if the
			# overall length matches (integrity check)
			if ((message[2] == 0x03) and (length == calc_len)):
				# This is a response and the length is correct
				return 1
			else:
				# Not a response or incorrect length
				return 0
		else:
			# Message is not long enough. Indicate an error
			return 0

	def __waitForMessage(self, timeout):
		"""
		This function simply waits for a message. It is like a read, but waits
		up to the specified timeout for new data to arrive
		Args:
			timeout: specified in seconds. The smallest delay period is 0.01s
		"""

		delay_time = 0.01
		# Force the timeout to be at least 10ms
		if (timeout < delay_time):
			timeout = delay_time
		num_iter = timeout / delay_time
		while(num_iter):
			try:
				num_bytes_in_buffer = self.port.numBytesInBuffer()

				if (num_bytes_in_buffer > 0):
					response = self.port.readPort(num_bytes_in_buffer)
			except Exception as e:
				self.log.error(e)
			else:
				if (num_bytes_in_buffer >= 5):
					return response
				else:
					num_iter = num_iter - 1
					time.sleep(delay_time)
		return None

	def __EvaluateAndReturn(self, response, expected_command, num_expected_payload_bytes):
		"""
		This function evaluates the response and prepares the API return value
		If 'num_expected_payload_bytes' is 0, then the response guides how
		many bytes will be returned
		"""

		if (response == None):
			return {'status': 0, 'errorCode': None, 'data': None}

		elif (self.__isResponse(response, expected_command,
										  num_expected_payload_bytes)):
			return {'status': 1, 'errorCode': None,
								 'data': response[-(
								 		 self.__getNumPayloadBytes(response)):]}

		elif (self.__isPushMessage(response, expected_command,
											 num_expected_payload_bytes)):
			return {'status': 1, 'errorCode': None,
								 'data': response[-(
								 		 self.__getNumPayloadBytes(response)):]}

		elif (self.__isACK(response)):
			return {'status': 1, 'errorCode': None, 'data': None}

		elif (self.__isNACK(response)):
			nack_value = self.__getNACKValue(response)
			return {'status': 0, 'errorCode': nack_value, 'data': None}

		else:
			return {'status': 0, 'errorCode': None, 'data': None}

	#********************
	#* PUBLIC FUNCTIONS *
	#********************

	def sendApiPortEnable(self):
		"""
		Sends a '*' to enable the API port which tells the probe to interact
		via the radicl API
		"""
		self.port.writePort([0x21])

	def Identify(self):
		"""
		Identifies the connected device
		Returns 1 if a valid device was detected
		"""

		ret = self.getHWID()
		self.hw_id = ret['data']
		time.sleep(0.5)
		ret = self.getHWREV()
		self.hw_rev = ret['data']
		time.sleep(0.5)
		ret = self.getFWREV()
		self.fw_rev = ret ['data']

		if (self.hw_id != None):
			if (self.hw_id < len(pca_id_list)):
				self.log.info("Attached device: %s, Rev=%s, FW=%s" % \
				  									(pca_id_list[self.hw_id],
													self.hw_rev,
													format(self.fw_rev, '.02f')))
				return 1

			else:
				self.log.info("Unknown device detected!")
				return 0

		else:
			self.log.error("Invalid response to ID request")
			return 0


	# ***************************
	# ***** BASIC COMMANDS ******
	# ***************************
	def getSerialNumber(self):
		"""
		Queries the board's serial number
		"""
		response = self.__send_receive([0x9F, 0x04, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x04, 8)

	def getHWID(self):
		"""
		Queries the board's HW ID
		"""

		response = self.__send_receive([0x9F, 0x01, 0x00, 0x00, 0x00])
		ret_val = self.__EvaluateAndReturn(response, 0x01, 1)
		if (ret_val['status'] == 1):
			byte_arr = ret_val['data']
			data = int.from_bytes(byte_arr, byteorder='little')
			ret_val['data'] = data

		return ret_val

	def getHWREV(self):
		"""
		Queries the board's HW revision
		"""

		response = self.__send_receive([0x9F, 0x02, 0x00, 0x00, 0x00])
		ret_val = self.__EvaluateAndReturn(response, 0x02, 1)
		if (ret_val['status'] == 1):
			byte_arr = ret_val['data']
			data = int.from_bytes(byte_arr, byteorder='little')
			ret_val['data'] = data

		return ret_val

	def getFWREV(self):
		"""
		Queries the board's FW revision
		"""

		response = self.__send_receive([0x9F, 0x03, 0x00, 0x00, 0x00])
		ret_val = self.__EvaluateAndReturn(response, 0x03, 2)
		if (ret_val['status'] == 1):
			value = ret_val['data']
			major = value[0] #value[-2]
			minor = value[1] #value[-1]
			fw_rev = major + minor/100
			ret_val['data'] = fw_rev
		return ret_val

	def startBootloader(self):
		"""
		Starts the bootloader
		"""

		response = self.__send_receive([0x9F, 0x05, 0x01, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x05, 0)

	def getSystemStatus(self):
		"""
		Queries the system status
		"""

		response = self.__send_receive([0x9F, 0x06, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x06, 4)

	def getRunState(self):
		response = self.__send_receive([0x9F, 0x07, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x07, 1)

	# ***************************************
	# ***** SENSOR/MEASUREMENT COMMANDS *****
	# ***************************************

	def getMeasState(self):
		"""
		Queries the state of the measurement state machine
		"""

		response = self.__send_receive([0x9F, 0x40, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x40, 1)

	def MeasReset(self):
		"""
		Resets the measurement state machine
		Returns 1 if successfull
		"""

		response = self.__send_receive([0x9F, 0x41, 0x01, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x41, 0)


	def MeasStart(self):
		"""
		Starts a measurement
		Returns 1 if successful
		"""

		response = self.__send_receive([0x9F, 0x42, 0x01, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x42, 0)


	def MeasStop(self):
		"""
		Stops a measurement
		Returns 1 if successfull
		"""

		response = self.__send_receive([0x9F, 0x43, 0x01, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x43, 0)

	def MeasGetNumSegments(self, buffer_id):
		"""
		Queries the number of data segments for a particular data buffer
		"""
		message = [0x9F, 0x44, 0x00, 0x00, 0x01]
		message.extend(buffer_id.to_bytes(1, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x44, 4)

	def MeasReadDataSegment(self, buffer_id, numPacket):
		"""
		Reads a specific data segment of a specific data buffer
		"""

		message = [0x9F, 0x45, 0x00, 0x00, 0x05]
		message .extend(buffer_id.to_bytes(1, byteorder='little'))
		message.extend(numPacket.to_bytes(4, byteorder='little'))
		response = self.__send_receive(message)

		# Check if only the command matches. The length may be variable
		return self.__EvaluateAndReturn(response, 0x45, 0)

	def MeasGetSamplingRate(self):
		"""
		Reads/Returns the IR sampling rate
		"""

		response = self.__send_receive([0x9F, 0x46, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x46, 4)

	def MeasSetSamplingRate(self, sampling_rate):
		"""
		Returns status=1 if successfull, status=0 otherwise

		helpme - Sets the sensor sampling rate
		"""

		message = [0x9F, 0x46, 0x01, 0x00, 0x04]
		message.extend(sampling_rate.to_bytes(4, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x46, 0)

	def MeasGetZPFO(self):
		"""
		Reads/returns the Zero Phase Filter Order used on the depth data.
		"""

		response = self.__send_receive([0x9F, 0x47, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x47, 4)

	def MeasSetZPFO(self, zpfo):
		"""
		Returns status=1 if successfull, status=0 otherwise

		helpme - Set the Zero Phase Filter Order used on the depth data.
		"""

		message = [0x9F, 0x47, 0x01, 0x00, 0x04]
		message.extend(zpfo.to_bytes(4, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x47, 0)

	def MeasGetPPMM(self):
		"""
		Reads/returns the Points per millimeter parameter
		"""

		response = self.__send_receive([0x9F, 0x48, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x48, 1)

	def MeasSetPPMM(self, ppmm):
		"""
		helpme - Sets the Points per millimeter parameter
		"""

		message = [0x9F, 0x48, 0x01, 0x00, 0x01]
		message.extend(ppmm.to_bytes(1, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x48, 0)

	def MeasGetALG(self):
		"""
		Reads/Returns the algorithm (1 - depth corrected, 2 for timeseries only)
	 	parameter
		"""

		response = self.__send_receive([0x9F, 0x49, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x49, 1)

	def MeasSetALG(self, alg):
		"""
		Returns status=1 if successfull, status=0 otherwise

		helpme - Sets the algorithm (1 - depth corrected, 2 for timeseries only)
		"""

		message = [0x9F, 0x49, 0x01, 0x00, 0x01]
		message.extend(alg.to_bytes(1, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x49, 0)

	def MeasGetAPPP(self):
		"""
		Reads the APPP parameter
		"""

		response = self.__send_receive([0x9F, 0x4A, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x4A, 1)

	def MeasSetAPPP(self, appp):
		"""
		Returns status=1 if successfull, status=0 otherwise

		helpme - Sets the APPP parameter which smooths the timeseries data

		"""

		message = [0x9F, 0x4A, 0x01, 0x00, 0x01]
		message.extend(appp.to_bytes(1, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x4A, 0)

	def MeasGetTCM(self):
		"""
		Reads the TCM parameter
		"""

		response = self.__send_receive([0x9F, 0x4B, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x4B, 1)

	def MeasSetTCM(self, tcm):
		"""
		Returns status=1 if successfull, status=0 otherwise

		helpme - Sets the Temperature correction method for the barometer data

		"""

		message = [0x9F, 0x4B, 0x01, 0x00, 0x01]
		message.extend(tcm.to_bytes(1, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x4B, 0)

	def MeasGetUserTemp(self):
		"""
		Reads the user set tempreature
		"""

		response = self.__send_receive([0x9F, 0x4C, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x4C, 4)

	def MeasSetUserTemp(self, user_temp):
		"""
		Returns status=1 if successfull, status=0 otherwise

		helpme - Set the user specifed temperature for TCM=3
		"""
		message = [0x9F, 0x4C, 0x01, 0x00, 0x04]
		message.extend(user_temp.to_bytes(4, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x4C, 0)

	def MeasGetIR(self):
		"""
		Reads the IR parameter

		"""
		response = self.__send_receive([0x9F, 0x4D, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x4D, 1)

	def MeasSetIR(self, ir):
		"""
		Returns status=1 if successfull, status=0 otherwise

		helpme - Turns on the IR emitter
		"""
		message = [0x9F, 0x4D, 0x01, 0x00, 0x01]
		message.extend(ir.to_bytes(1, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x4D, 0)

	def MeasGetCalibData(self, num_sensor):
		"""
		Reads a sensor's calibration value
		"""

		message = [0x9F, 0x4E, 0x00, 0x00, 0x01]
		message.extend(num_sensor.to_bytes(1, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x4E, 4)

	def MeasSetCalibData(self, num_sensor, calibration_value):
		"""
		Returns status=1 if successfull, status=0 otherwise

		helpme - Sets the calibration data for the specified sensor
		"""

		message = [0x9F, 0x4E, 0x01, 0x00, 0x05]
		message.extend(num_sensor.to_bytes(1, byteorder='little'))
		message.extend(calibration_value.to_bytes(4, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0x4E, 0)

	def MeasGetMeasTemp(self):
		"""
		Reads the current temperature reading (from last measurement)
		Returns status=1 if successfull, status=0 otherwise
		"""

		response = self.__send_receive([0x9F, 0x4F, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0x4F, 0)

	# ******************************
	# ***** FW UPDATE COMMANDS *****
	# ******************************

	def UpdateEnter(self):
		"""
		Enters the FW Update FSM
		Returns status=1 if successfull, status=0 otherwise
		"""

		response = self.__send_receive([0x9F, 0xF0, 0x01, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0xF0, 0)

	def UpdateGetState(self):
		"""
		Gets the FW update FSM state
		"""

		response = self.__send_receive([0x9F, 0xF1, 0x00, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0xF1, 1)

	def UpdateWaitForStateChange(self, wait_time):
		"""
		Waits for the state change message
		"""

		response = self.__waitForMessage(wait_time)
		return self.__EvaluateAndReturn(response, 0xF1, 1)

	def UpdateSetSize(self, num_packets, packet_size):
		"""
		Sets the update size
		Returns status=1 if successfull, status=0 otherwise
		"""

		message = [0x9F, 0xF2, 0x01, 0x00, 0x06]
		message.extend(num_packets.to_bytes(4, byteorder='little'))
		message.extend(packet_size.to_bytes(2, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0xF2, 0)

	def UpdateDownload_Short(self, data, crc8, packet_id):
		"""
		Downloads a 16-byte chunck of data
		Returns status=1 if successfull, status=0 otherwise
		"""

		message = [0x9F, 0xF3, 0x01, 0x00, 0x00]
		message[3] = crc8
		message[4] = 16
		message.extend(data)
		self.__sendCommand(message)
		response = self.__waitForMessage(20)
		return self.__EvaluateAndReturn(response, 0xF3, 0)

	def UpdateDownload_Long(self, data, crc8, packet_id):
		"""
		Downloads a 256-byte chunck of data
		Returns status=1 if successfull, status=0 otherwise
		"""

		message = [0x9F, 0xF3, 0x07, 0x00, 0x00]
		message[3] = crc8
		message[4] = 0
		message.extend(data)
		self.__sendCommand(message)
		response = self.__waitForMessage(20)
		return self.__EvaluateAndReturn(response, 0xF3, 0)

	def UpdateSetCRC(self, crc32):
		"""
		Sets the CRC32 of the FW image
		Returns status=1 if successfull, status=0 otherwise
		"""

		message = [0x9F, 0xF4, 0x01, 0x00, 0x04]
		message.extend(crc32.to_bytes(4, byteorder='little'))
		response = self.__send_receive(message)
		return self.__EvaluateAndReturn(response, 0xF4, 0)

	def UpdateClose(self):
		"""
		Closes the FW update FSM
		Returns status=1 if successfull, status=0 otherwise
		"""

		response = self.__send_receive([0x9F, 0xF5, 0x01, 0x00, 0x00])
		return self.__EvaluateAndReturn(response, 0xF5, 0)



# coding: utf-8

import os
import time
from radicl.serial import RadConnection, MessageColors, ProbeCommand

out = MessageColors()

def calibrate(probe, value_str):

    # raw_input(out.msg("Press enter to start a reading:"))
    # probe.send('meas start')
    # raw_input(out.msg("Press enter to stop a reading:"))
    # probe.send('meas stop')
    probe.take_a_reading(method = 'keyboard', timed = 1)

    cmd = ProbeCommand('fDump')
    try:
        result = probe.get_data(cmd)
    except:
        out.msg('ERROR: Failed to retrieve data. Exiting...','FAIL')
        probe.close()
        sys.exit()
    calib = []
    i = 1
    for col in result[1:]:
        if value_str == 'max':
            value = max(col)
        else:
            value = min(col)
        out.msg("Detected Sensor {1} calibration value = {0}".format(value,i))
        i+=1
        calib.append(value)

    time.sleep(0.25)
    probe.reset_probe()

    return calib

def main():
    if 'nt' in os.name:
        os.system('cls')  # for Windows
    else:
        os.system('clear')  # for Linux/OS X
    out.msg(out.BOLD + "-------------LYTE Probe Calibrator-------------",'HEADER')
    out.msg("USE:",'BOLD')
    out.msg("\tCalibrator will first take a reading\n\tto calibrate the high value then the\n\tlow value. Use targets to accomplish\n\tthe values for each high and low.")
    out.msg(out.BOLD + "----------------------------------------------",'HEADER')

    probe = RadConnection()
    out.msg("\nCover sensors using the high value producing target.")
    hi = calibrate(probe,"max")
    out.msg("\nCover sensors using the low value producing target.")
    low = calibrate(probe,"min")

    for i in range(4):
        if low[i] > hi[i]:
            out.msg('ERROR: Sensor {0}s Low calibration value is greater than the high calibration value.'.format(i),'FAIL')
        else:
            out.msg('\nApplying calibration values to sensor {0}'.format(i))
            probe.send('meas setCalib {0} {1} {2}'.format(i,low[i],hi[i]))

            probe.listen_for('calibration')
            out.msg("\n\tSensor {0} calibrated!".format(i),'OKGREEN')

if __name__ == '__main__':
    main()



# coding: utf-8

import sys
import time
from matplotlib import pyplot as plt
import os
from termcolor import colored
import inspect
import pandas as pd
from radicl.ui_tools import Messages, parse_func_list, print_helpme, parse_help
import datetime

from radicl import probe
from radicl import serial
from radicl import api
import radicl

out = Messages()

class RADICL:
    def __init__(self,**kwargs):

        self.tasks = ['daq','settings','update']
        self.task_help = {'daq':"Data acquisition using the probe.",
                          'settings':'Interface for modifying the behavior of the probe',
                          'update': 'Firmware update dialog for the probe.'}

        self.probe = probe.RAD_Probe()
        self.running = True

        self.settings = dir(self.probe.api)

        # Get all the functions available in api and probe
        settings_funcs = inspect.getmembers(self.probe.api, predicate=inspect.ismethod)
        data_funcs = inspect.getmembers(self.probe, predicate=inspect.ismethod)
        # Dictionary of the options available that can be auto parsed
        # Each option type has keys word that searched for in the api and probe classes
        self.options_keywords = {'data':{'parseable':['read','Data']},
                        'settings':{'parseable':['Meas','Set']}
                        }
        self.options = {'data':{},
                       'settings':{}}
        # Assign all functions with keys word to auto gather settings and data packages
        self.options['data'] = parse_func_list(data_funcs,['read','Data'],
                                             ignore_keywords = ['correlation'])
        self.options['settings'] = parse_func_list(settings_funcs,['Meas','Set'])
        self.options['settings']["show"] = self.print_settings

        self.options['getters'] = parse_func_list(settings_funcs,['Meas','Get'])

        # User runtime preferences
        self.output_preference = None
        self.filename = None
        self.daq = None


        # Gather help dialogs
        self.help_dialog= {}

        for s in self.options.keys():
            # We haven't added the category yet
            if s not in self.help_dialog:
                self.help_dialog[s] = {}

            # Add in all the functions
            for k,fn in self.options[s].items():
                self.help_dialog[s][k] = parse_help(fn.__doc__)

    def run(self):
        """
        Main running loop for the command line Interface
        """

        self.state = 0
        plot = False
        write = False
        both = False
        filename_used = False
        data_attempts = 1

        # Create the probe command with an arbitrary command to have access to help
        while self.running:
            pstate = self.probe.getProbeMeasState()

            if self.state == 0:
                #If probe is not ready.
                if pstate != 0:
                    pstate = self.probe.resetMeasurement()

                else:
                    #Request type of operation
                    self.task = self.ask_user("What do you want to do with the"
                                                      " probe?", self.tasks,
                                                      helpme = self.task_help)

            elif self.state >= 1:
                self.tasking()

            else:
                pass

    def tasking(self):
        """
        Handles tasks from the user, specifically any member function who
        contains the keyword task_ in it. This means currently we are looking
        for a daq, settings, upgrade task function.
        """
        # Auto look for task_functions
        task_at = "task_%s" % self.task
        fn = getattr(self,task_at)

        # Call task_function
        out.dbg("Using task function {0}".format(fn.__name__))
        fn()


    def take_a_reading(self,data_request):

        input("Press any key to begin a measurement.\n")
        response = self.probe.startMeasurement()

        input("Press any key to stop the measurement.\n")
        response = self.probe.stopMeasurement()


        fn = self.options['data'][data_request]
        out.dbg("Requesting data using function {0}".format(fn.__name__))

        self.probe.wait_for_state(3)

        temp = self.probe.readMeasurementTemperature()
        if (temp != None):
            out.dbg("Temp = %i" % temp)
        else:
            out.error("Error reading temp")

        data = fn()

        response = self.probe.resetMeasurement()
        data = self.dataframe_this(data,data_request)

        return data

    def dataframe_this(self,data,name):
        """
        Takes the data returned from a function and converts it into a DataFrame
        There appears to be two scenarios so we cover it here.
        """
        t = type(data)
        if t == dict:
            data = pd.DataFrame(data, columns = data.keys())

        elif t == list:
            if type(data[0])==tuple:
                data = [d[0] for d in data]

            data = pd.DataFrame(data, columns = [name])
        return data


    def task_daq(self):
        """
        The state machine for request to do daq
        """
        # Data Selection
        if self.state==1:
            pstate = self.probe.getProbeMeasState()
            out.msg("Checking to see if probe is ready...")
            out.dbg("Probe State = {0}".format(pstate))

            # Make sure probe is ready
            if pstate==0 or pstate==5:
                self.daq = self.ask_user("What data do you want?",
                                    list(self.options['data'].keys()),
                                    helpme = self.help_dialog['data'])

        # Method for outputting data
        elif self.state == 2:
            self.output_preference = self.ask_user("How do you want to output"
                                       " {0} data?".format(self.daq),
                                       ['plot','write','both'],
                                       default_answer = self.output_preference)

        # Take Measurements
        elif self.state == 3:
            self.data = self.take_a_reading(self.daq)
            self.state = 4

        # Data output and options
        elif self.state == 4:
            if self.output_preference == 'write' or self.output_preference == 'both':
                valid = False
                #  Wait for real path
                while not valid:
                    t = datetime.datetime.now()
                    fstr = "{0}-{1:02d}-{2:02d}--{3:02d}{4:02d}{5:02d}"
                    fname = fstr.format(t.year,t.month,t.day,t.hour,t.minute,t.second)
                    out.msg("Enter in a filepath for the data to be saved:")
                    filename = input("\nPress enter to use default:"
                                    "\n(Default: ./{0}.csv)".format(fname))

                    # Recieve a default request
                    if filename == '':
                        filename = os.path.expanduser('./{0}.csv'.format(fname))
                    else:
                        filename = os.path.expanduser(filename)
                    # Make it absolute
                    filename = os.path.abspath(filename)
                    real_path = os.path.isdir( os.path.dirname(filename))

                    # Double check a real path was given
                    if not real_path:
                        out.warn("Path provided does not exist.")
                        valid = False
                    else:
                        self.filename = filename
                        valid = True
                        out.msg("Saving Data to :\n{0}".format(self.filename))

                        if not self.data.empty:
                            # Write the header so we knwo things about this
                            with open(self.filename,'w') as fp:
                                final = self.probe.getProbeHeader()
                                fp.writelines(final)
                                fp.close()
                            # Write data
                            self.data.to_csv(self.filename, mode = 'a')
                            self.state = 5

            if self.output_preference == 'plot' or self.output_preference =='both':
                self.data.plot()
                plt.show()
                self.state = 5

        elif self.state==5:
            pos_ans = self.ask_user("Take another measurement?")
            if pos_ans:
                self.state = 3
            elif not pos_ans:
                self.state = 1


    def task_settings(self):
        """
        Routine for handling users requests for modifying probe settings settings
        """

        # Data Selection
        if self.state == 1:
            pstate = self.probe.getProbeMeasState()
            out.msg("Checking to see if probe is ready...")
            out.dbg("Probe State = {0}".format(pstate))

            # Make sure probe is ready
            if pstate == 0 or pstate == 5:
                print( self.help_dialog['settings'])
                self.setting_request = self.ask_user("What settings do you want"
                                                     " to adjust?",
                               list(self.options['settings'].keys()),
                                helpme = self.help_dialog['settings'])

        # Get current setting
        elif self.state == 2:

            if self.setting_request == 'calibdata':
                out.error("Calibration method are under developed at this time")
                self.state = 1

            elif self.setting_request == 'show':
                self.options['settings'][self.setting_request]()
                self.state = 1

            else:
                self.current_setting_value = \
                        self.probe.getSetting(setting_name=self.setting_request)
                self.state += 1

        # Set settings
        elif self.state == 3:

                if self.setting_request == 'calibdata':
                    out.error("Calibration method are under developed at this time")
                    self.state = 1

                else:
                    msg = ("Currently {0} = {1}\nEnter value to change probe {0}\n"
                           "".format(self.setting_request,self.current_setting_value))
                    valid = False

                    # All entries have to be numeric. Ensure this is the case.
                    while not valid:
                        self.new_value = input(msg)
                        try:
                            self.new_value = int(self.new_value)
                            valid = True
                        except Exception as e:
                            print(e)
                            out.error("Value must be numeric!")

                    # Call the function to change the setting
                    self.probe.setSetting(setting_name = self.setting_request,
                                          value = self.new_value)
                    time.sleep(0.2)

                    # Confirm the value was changed
                    test_value = self.probe.getSetting(
                                            setting_name = self.setting_request)

                    if test_value == self.new_value:
                        out.respond("{0} was changed from {1} to {2}!\n"
                                    "".format(self.setting_request,
                                              self.current_setting_value,
                                              self.new_value))
                    # Go back to settings menu
                    self.state = 1

    def print_settings(self):
        """
        print all the current settings

        helpme - Prints out all the settings fro the probe

        """

        msg = "\n===== Current Probe Settings =====\n"

        for s, fn in self.options['getters'].items():

            if s.lower() not in ['calibdata','numsegments']:
                value = self.probe.getSetting(setting_name=s)
                msg += "{} = {}\n".format(s, value)
        out.msg(msg)
        time.sleep(2)

    def increment_fnumber(self,filename):
        """
        Check for a file numbering. Increment if there is one. Otherwise add one
        """
        if self.is_numbered(filename):
            fcount = int(filename[-1])+1

        else:
            fcount = 1

        s = -1*len(str(fcount)) + pos_num
        filename = filename[0:s] + str(fcount)

        return filename

    def is_numbered(self, filename):
        """
        Checks if the filename is numbered. Returns true or false.
        """
        if filename[-4] in [str(i) for i in range(0,9)]:
            result = True

        else:
            result = False

        return result

    def check_ext(self,filename):
        """
        Check to see if the user provided the .csv ext in the filename
        and add it
        """
        if filename[-4:] != '.csv':
            f = filename.split('.')
            # Did the user try to add an ext
            if len(f) == 2:
                filename = f[0]+'.csv'
            else:
                filename +='.csv'

        return filename


    def ask_user(self,question_str, answer_lst = None, helpme = None, next_state = True, default_answer = None):
        """
        Function is used to wait for an appropriate response from user and handle unknown answers
        """

        # Ask user for a yes no question
        if answer_lst == None:
            answer_lst = ['y','n']
            question_boolean = True
        else:
            question_boolean = False

        # Keep question to one line if small number of options
        lower_lst = [i.lower() for i in sorted(answer_lst)]
        lower_lst.append('exit')
        lower_lst.append('home')

        # Help documentation
        if helpme != None:
            lower_lst.append("help")

        if len(lower_lst) <= 5:
            print_able=" ("
            print_able += ", ".join(lower_lst)
            print_able+=")"
        # Create a column of choices
        else:
            print_able='\n{0}'.format(colored('[OPTIONS]','magenta',attrs=['bold']))
            for s in lower_lst:
                print_able+= '\n  {0}'.format(s)

        question_str+=print_able

        # Provide prompt for default options
        if default_answer != None:
            if default_answer not in answer_lst:
                raise ValueError("default answer must be a member of the acceptable answers list in ask_user function")

            question_str += '(Default: {0})'.format(default_answer)

        out.msg(question_str)

        acceptable_answer=False

        while acceptable_answer==False:
            user_answer = input('\n')
            response = user_answer.lower()

            # User requests help with options
            if "help" in response:
                print_helpme(response,helpme)
                acceptable_answer=False
                out.msg(question_str)

            # Request to leave the program
            elif response=='exit':
                out.respond("Exiting the RAD CLI.")
                self.running = False
                sys.exit()

            elif response=='home':
                out.respond("Returning to the main menu.")
                self.running = True
                acceptable_answer=True
                self.state = 0

            # Default Picked
            elif response == '':
                response = default_answer

            # User provided a potential acceptable answer
            elif response in lower_lst:
                acceptable_answer=True
                response = lower_lst[lower_lst.index(response)]
                if question_boolean:
                    if response == 'y' or response =='yes':
                        response = True
                    else:
                        response = False

                if next_state:
                    self.state +=1


            else:
                out.error('Only acceptable answers are:')
                out.msg(print_able)
                acceptable_answer=False

        return response



# coding: utf-8

import time
import binascii
import hashlib
from .ui_tools import get_logger

class FW_Update():

	#*********************
	#* PRIVATE FUNCTIONS *
	#*********************

	def __init__(self, api, packet_size):
		self.f = None
		self.api = api
		self.file_crc = 0
		self.packet_size = packet_size
		self.log = get_logger(__name__, level='DEBUG')
	def __getFileSize(self):
		"""
		Returns the file size of the specified file
		"""

		self.f.seek(0,2) # move the cursor to the end of the file
		size = self.f.tell()
		return size

	def __swapBytesInWord(self, fourBytes):
		"""
		Helper function to swap a word (4 bytes)
		"""

		return [fourBytes[3], fourBytes[2], fourBytes[1], fourBytes[0]]

	def __swapBytesInArray(self, byteArray):
		"""
		Helper function to swap bytes in an array (LSByte to MSByte)
		"""

		length = len(byteArray)
		swappedArray = bytearray(length)
		index = 0
		offset = 0
		four_count = 4
		for b in byteArray:
			swappedArray[offset + four_count - 1] = byteArray[index]
			index += 1
			four_count = four_count - 1
			if (four_count == 0):
				offset = index
				four_count = 4
		return swappedArray

	def __LoadFileHeader(self):
		"""
		Loads/reads the file header
		"""

		self.f.seek(2048, 0)
		bytes = self.f.read(4)
		hex_string1 = "".join("%02x" % b for b in self.__swapBytesInWord(bytes))
		bytes = self.f.read(4)
		hex_string2 = "".join("%02x" % b for b in self.__swapBytesInWord(bytes))

		if (hex_string1 == "dabbad00" and hex_string2 == "a5b6c7d8"):
			bytes = self.f.read(4)
			hex_string = "".join("%02x" % b for b in self.__swapBytesInWord(bytes))

			self.app_header_version = int(hex_string, 16)
			bytes = self.f.read(4)
			hex_string = "".join("%02x" % b for b in self.__swapBytesInWord(bytes))

			self.app_header_length = int(hex_string, 16)
			bytes = self.f.read(4)
			hex_string = "".join("%02x" % b for b in self.__swapBytesInWord(bytes))

			self.app_header_entry = int(hex_string, 16)
			bytes = self.f.read(4)
			hex_string = "".join("%02x" % b for b in self.__swapBytesInWord(bytes))

			self.app_header_crc = int(hex_string, 16)
			return 1

		# Header magic words did not match.
		else:
			return 0

	def __DumpHeaderInfo(self):
		"""
		Dumps the header info
		"""

		self.log.info("*** Header Info ***"
			  "\nVersion\t=\t%d"
			  "\nApp length =\t%d"
			  "\nEntry\t=\t0x%0.8X"
			  "\nCRC\t=\t0x%0.8X" % (self.app_header_version,
			  						 self.app_header_length,
									 self.app_header_entry,
									 self.app_header_crc))
		return 0

	def __CalculateAppCRC(self):
		self.f.seek(self.app_header_entry, 0)	#Set the file pointer to the beginning of the application section
		app_bytes = self.f.read(self.app_header_length)	#Read the entire application section from the file
		byte_string = "".join("%02x" % b for b in self.swapBytesInWord(app_bytes))
		self.log.debug("CRC non-swapped =\t0x%.08X, 0x%.08X" % (binascii.crc32(app_bytes, 0x4C11DB7), (0x100000000 - binascii.crc32(app_bytes, 0x4C11DB7))))
		self.log.debug("CRC swapped =\t0x%.08X, 0x%.08X" % (binascii.crc32(self.__swapBytesInArray(app_bytes), 0x4C11DB7), (0x100000000 - binascii.crc32(self.__swapBytesInArray(app_bytes), 0x4C11DB7))))

		# return sum(app_bytes)
		return binascii.crc32(app_bytes)

	def __CalculateImageCRC32(self):
		"""
		Calculates the CRC32 of the entire FW image file
		"""

		file_len = self.__getFileSize()
		self.f.seek(0, 0)
		data = self.f.read(file_len)
		return binascii.crc32(data)

	def __CalculateChecksum(self, data):
		"""
		Calculates the checmsum of the supplied data package
		"""

		s = sum(data)
		return (s % 256)

	def __calculateNumDataPackets(self):
		"""
		Calculates the number of data packets based on the file size
		"""

		file_size = self.__getFileSize()
		num_packets = file_size / self.packet_size
		self.num_packets = num_packets
		#self.num_packets = 16
		return num_packets

	#********************
	#* PUBLIC FUNCTIONS *
	#********************

	def loadFile(self, file_name):
		"""
		Loads and opens a FW update file
		This also verifies the file integrity and CRC
		Returns 1 if operation succeeded
		"""

		try:
			self.f = open(file_name, "rb")
		except Exception as e:
			self.log.error (e)
			self.f = None
			return 0
		else:
			self.file_crc = self.__CalculateImageCRC32()
			self.log.debug("File CRC32 = 0x%08X" % self.file_crc)
			hash_md5 = hashlib.md5()
			self.f.seek(0,0)
			for chunk in iter(lambda: self.f.read(4096), b""):
				hash_md5.update(chunk)
			self.log.debug("MD5 = %s" % hash_md5.hexdigest())

			# If we get here then the file is valid and can be used
			return 1

	def closeFile(self):
		"""
		Closes the file
		"""

		self.f.close()
		self.f = None

	def getState(self):
		"""
		Returns the current state of the update FSM
		Returns the state if successful, None otherwise
		"""
		ret = self.api.UpdateGetState()
		if (ret['status'] == 1):
			byte_arr = ret['data']
			return int.from_bytes(byte_arr, byteorder='little')
		if (ret['errorCode'] != None):
			self.log.error("FW_Update.getState returned error %d" % ret['errorCode'])
		return None

	def waitForStateChange(self, timeout):
		"""
		Waits for an update FSM state change message
		Returns the new state if successfull, None if timeout or incorrect
		response
		"""

		ret = self.api.UpdateWaitForStateChange(timeout)

		if (ret['status'] == 1):
			byte_arr = ret['data']
			return int.from_bytes(byte_arr, byteorder='little')

		# Retry in case of timeout
		else:
			self.log.debug("Timeout - Retry!")
			return self.getState()
		if (ret['errorCode'] != None):
			self.log.error("FW_Update.waitForStateChange returned error %d" % \
				  ret['errorCode'])
		return None

	def closeFSM(self):
		"""
		Closes the update FSM (i.e. reset)
		Returns 1 if successfull, 0 otherwise
		"""

		ret = self.api.UpdateClose()
		if (ret['status'] == 1):
			return 1

		if (ret['errorCode'] != None):
			self.log.error("FW_Update.closeFSM returned error %d" % ret['errorCode'])
		return 0

	def enterFSM(self):
		"""
		Enter the FW update FSM (starts the process)
		Returns 1 if successfull, 0 otherwise
		"""

		ret = self.api.UpdateEnter()

		if (ret['status'] == 1):
			self.log.info("Enter FSM - Waiting for state change")
			state = self.waitForStateChange(10)

			if (state == None):
				self.log.info("Timeout: No response")
				return 0

			else:
				if (state == 1):
					self.log.info("Waiting for preparation stage to finish...")
					state = self.waitForStateChange(10)
					if (state == None):
						self.log.info("Timeout: No response")
						return 0

				if (state == 2):
					self.log.info("FSM Enter Done!")
					return 1
				self.log.error("Error: Incorrect state (%d)" % state)
				return 0

		if (ret['errorCode'] != None):
			self.log.error("FW_Update.enterFSM returned error %d" % ret['errorCode'])
		self.log.error("Enter FSM - Failed!")
		return 0

	def sendNumPackets(self):
		"""
		Sends the number of data packets and ensures we advance to the next step
		Returns 1 if successfull, 0 otherwise
		"""
		state = self.getState()

		if (state == 2):
			self.__calculateNumDataPackets()
			self.log.debug("Total number of packets = %d" % self.num_packets)
			ret = self.api.UpdateSetSize(int(self.num_packets),
										 self.packet_size)
			if (ret['status'] != 1):
				self.log.error("Error setting number of packets")
				if (ret['errorCode'] != None):
					self.log.error("FW_Update.sendNumPackets returned error %d" % \
					 	  ret['errorCode'])
				return 0

			time.sleep(1)
			state = self.waitForStateChange(5)

			if (state == None):
				self.log.info("Timeout: No response")
				return 0

			else:
				self.log.info("New state = %s" % state)
				if (state == 3):
					return 1
				return 0
		else:
			if (state == None):
				self.log.error("Error: Incorrect or no response from device...")
			else:
				self.log.error("Error: Incorrect state (%d)" % state)
			return 0

	def downloadFile(self):
		"""
		Downloads the file in chuncks
		Returns 1 if successfull, 0 otherwise
		"""
		state = self.getState()
		if (state == 3):
			#Set the file pointer to the very beginning
			self.f.seek(0)
			packet_counter = 0
			retry_count = 0
			packet_id = 0

			for ii in range(0, int(self.num_packets)):
				data = self.f.read(self.packet_size)
				packet_id_arr = packet_id.to_bytes(4, byteorder='little')
				retry = 1
				retry_count = 0

				while(retry == 1):
					crc8 = self.__CalculateChecksum(data)
					if (self.packet_size == 256):
						ret = self.api.UpdateDownload_Long(data,
														   crc8,
														   packet_id_arr[0])
					else :
						ret = self.api.UpdateDownload_Short(data,
															crc8,
															packet_id_arr[0])

					# Packet was not accepted. Check if it was checksum mismatch
					if (ret['status'] != 1):

						# Checksum mismatch detected.
						if (ret['errorCode'] == 5121):
							if (retry_count > 3):
								self.log.info ("Max retries exceeded. Stopping.")
								return 0
							# Retry, by simply allowing to loop again
							retry = 1
							retry_count = retry_count + 1
							self.log.error("Checksum mismatch error. Retrying packet"
								  " %d (%d)" % (packet_id, retry_count))

						# Error other than checksum mismatch.
						else:
							self.log.error("Download error (%d, Packet=%d)" % \
																(ii, packet_id))
							return 0

					# Packet was accepted by device. Skip retry/ update progress
					else:
						retry = 0
						pct = (100 * ii)/self.num_packets
						self.log.info("Progress = %d%%    " % pct, end="\r", flush=True)
						packet_id = packet_id + 1
						retry_count = 0

			# If we get here then the entire download succeeded
			self.log.info("Download done - Waiting for state change")
			time.sleep(1)
			state = self.waitForStateChange(5)
			if (state == None):
				self.log.warning("Timeout: No response")
				self.log.info("Manually checking if state has changed")
				state = self.getState()
				self.log.info("New state = %d" % state)
				if (state == 4):
					return 1
				else:
					return 0
			return 1
		else:
			if (state == None):
				self.log.error("Error: Incorrect or no response from device...")
			else:
				self.log.error("Error: Incorrect state (%d)" % state)
			return 0

	def sendImageCRC32(self):
		"""
	 	Sends the CRC32 of the entire FW image
		Returns 1 if successfull, 0 otherwise
		"""

		# Check that we are in the right state
		state = self.getState()

		if (state == 4):
			ret = self.api.UpdateSetCRC(self.file_crc)

			if (ret['status'] != 1):
				self.log.error("Error sending CRC32")

				if (ret['errorCode'] != None):
					self.log.error("FW_Update.sendImageCRC32 returned error %d" % \
					 										   ret['errorCode'])
				return 0
			time.sleep(1)
			state = self.waitForStateChange(5)

			if (state == None):
				self.log.warning("Timeout: No response")
				return 0

			if (state == 5):
				self.log.debug("CRC32 set - Waiting for verification to complete")
				#upd_state = self.api.UpdateWaitForStateChange(2000)
				state = self.waitForStateChange(20)

				if (state == None):
					self.log.warning("Timeout: No response")
					return 0

				if (state == 6):
					self.log.info("Verification complete")
					return 1

				else:
					self.log.error("Verification failed. New state = %d" % state)
					return 0

			elif (state == 6):
				self.log.debug("CRC32 set!")
				self.log.info("Verification complete!")
				return 1

			else:
				self.log.error("Setting CRC32 failed. New state = %d" % state)
				return 0

		else:

			if (state == None):
				self.log.error("Error: Incorrect or no response from device...")
			else:
				self.log.error("Error: Incorrect state (%d)" % state)
			return 0

	def upgrade(self):
		"""
		Upgrades the FW on the device
		"""

		self.log.info("*** FW UPDATE PROCESS STARTED ***")
		# First, check if we need to reset the FSM
		state = self.getState()

		if (state == None):
			self.log.error("Error. No response")
			return 0

		elif (state != 0):
			self.log.warning("FSM not in IDLE. Resetting...")
			if (self.closeFSM() != 1):
				self.log.error("Error: Closing failed")
				return 0

		self.log.info("Step 1: Enter FSM")
		if (self.enterFSM() != 1):
			self.log.error("Error: Could not enter FW Update FSM");
			return 0

		self.log.info("Step 2: Send number of packets")
		if (self.sendNumPackets() != 1):
			self.log.error("Error: Could not set number of packets")
			return 0

		self.log.info("Step 3: Downloading data")
		if (self.downloadFile() != 1):
			self.log.error("Error: Download failed")
			return 0

		self.log.info("Step 4: Setting CRC")
		if (self.sendImageCRC32() != 1):
			self.log.error("Error: Sending and verifying CRC32 failed")
			return 0

		self.log.info("Step 5: Close")
		if (self.closeFSM() != 1):
			self.log.error("Error: Closing failed")
			return 0

		self.log.info ("FW Update complete. Device will disconnect and reboot")

		return 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst', encoding='utf-8') as readme_file:
    readme = readme_file.read()

with open('docs/history.rst', encoding='utf-8') as history_file:
    history = history_file.read()

with open('./requirements_dev.txt', encoding='utf-8') as req_file:
    requirements = req_file.read()

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="Micah Johnson",
    author_email='micah@adventuredata.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Command line interface to the Realtime Adventure Data Lyte probe for measuring avalanche conditions",
    install_requires=requirements,
    license="BSD license",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    keywords='radicl',
    name='radicl',
    packages=find_packages(include=['radicl']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/adventuredata/radicl',
    version='0.2.4',
    zip_safe=False,
    scripts = ['scripts/plotlyte',],
     entry_points = {
          'console_scripts': [
              'radicl = radicl.cli:main',
          ],
      },
)



# coding: utf-8

import sys
import serial
from serial.tools import list_ports
from .ui_tools import get_logger

class RAD_Serial():
	"""
	This class handles all serial communication and simply acts
	as an abstraction layer
	"""

	def __init__(self):
		self.serial_port = None
		self.log = get_logger(__name__, level='DEBUG')

	def openPort(self, com_port=None):

		# No COM port has been provided. Need to detect port automatically
		if (com_port == None):
			self.log.info("No COM port provided. Scanning for COM ports...")
			match_list = list()

			# Run through all available COM ports and filter out the ones that match our requirement
			port_list = list_ports.comports()
			for p in port_list:
				if ('STMicroelectronics' in p[1] or 'STM32' in p[1]):
					match_list.append(p)

			#Throw an exception if there are no ports
			if not match_list:
				raise IOError("No COM ports found")

			#Check if more than one was found. If so, simply self.log.info a message
			if(len(match_list) > 1):
				self.log.info('Multiple COM ports found - using the first')

			#Finally, assign the found port to the serial_port variable
			this_p = match_list[0]
			try:
				self.serial_port = serial.Serial(port=this_p[0],
												 baudrate=115200,
												 parity=serial.PARITY_NONE,
                                        		 stopbits=serial.STOPBITS_ONE,
												 bytesize=serial.EIGHTBITS,
												 timeout=0.01,
												 write_timeout=0,
												 xonxoff=False,
												 rtscts=False,
												 dsrdtr=False)
				self.serial_port.setDTR(1)

			except Exception as e:
				self.log.error("Serial port open failed: %s" %e)
				self.serial_port = None
				raise IOError("Could not open port %s" % com_port.port)
			self.log.info("COM port found. Using %s" % self.serial_port.port)

		else:
			try:
				self.serial_port = serial.Serial(port=com_port,
												 baudrate=115200,
												 parity=serial.PARITY_NONE,
                                        		 stopbits=serial.STOPBITS_ONE,
												 bytesize=serial.EIGHTBITS,
												 timeout=0.01,
												 write_timeout=0,
												 xonxoff=False,
												 rtscts=False,
												 dsrdtr=False)
				self.serial_port.setDTR(1)

			except Exception as e:
				self.log.error("Serial port open failed: %s" % e)
				self.serial_port = None
				raise IOError("Could not open port %s" % com_port)

			self.log.info("Using %s" % serial_port.port)

	def closePort(self):
		if (self.serial_port != None):
			self.serial_port.close

	def flushPort(self):
		if (self.serial_port != None):
			self.serial_port.flushOutput()

	def writePort(self, data):
		"""
		Writes data to the serial port
		"""

		if (self.serial_port != None):
			return self.serial_port.write(serial.to_bytes(data))

	def writePortClean(self, data):
		"""
		Writes data to the serial port, but first clears the input buffer if
		there is data. This ensures that the next read will be alligned with
		the response from this write
		"""

		if (self.serial_port != None):
			if (self.self.numBytesInBuffer() > 0):
				self.self.flushPort()

			return self.serial_port.write(serial.to_bytes(data))

	def readPort(self, numBytes=None):
		if (self.serial_port != None):
			if (numBytes == None):
				# No number of specifed bytes to read. Read all available bytes
				return self.serial_port.read(self.serial_port.inWaiting())
			else:
				return self.serial_port.read(numBytes)

	def numBytesInBuffer(self):
		if (self.serial_port != None):
			return self.serial_port.inWaiting()
		return 0
